const express = require("express")
const mqtt = require('mqtt')
const orderRoutes = require("./routes/orderRoutes");
const cabinetRoutes = require("./routes/cabinetRoutes");
const cors = require("cors");
const resources = require('./resources');

// Express
const app = express();
app.use(express.json({ limit: "10MB" }));

const corsOptions = {
origin: "*",
credentials: true, //access-control-allow-credentials:true
optionSuccessStatus: 200,
};
app.use(cors(corsOptions));

app.get('/cabinet',(req,res)=>{
    return res.send(cabinetRoutes.get_cabinet());
})

app.get('/order',(req,res)=>{
    return res.send(orderRoutes.order());
})

app.post('/order/submit',(req,res)=>{
    client.publish('/test/number1',JSON.stringify({"order_id": 1,"control":1}))
    return res.send(
        orderRoutes.order_submit({
            "order" : req.body.order
        })
    );
})


let awaiting_release = -1

// MQTT
const mqtt_routes = ['/cabinet/keypad','/cabinet/1/sensor','/cabinet/2/sensor','/cabinet/detect']
console.log("Connecting to mqtt...")
const client = mqtt.connect('mqtt://broker.emqx.io:1883',{
    clean: true,
    connectTimeout: 30000,
    reconnectPeriod: 30000,
  })

client.on('connect',function(){
    console.log("Connecting to mqtt...")
    client.subscribe(mqtt_routes, function (err) {
        if (err) {
            console.log(err);
        }
        console.log("Connected to mqtt")
    });
})

client.on('message', function (topic, message) {
    const route = topic.toString();
    const req = JSON.parse(message.toString());
    console.log(route,req)
    switch(route){
        case '/cabinet/keypad':
            if(req.cabinet == 1){
                client.publish('/cabinet/1',JSON.stringify({"release":1}))
            }
            else if(req.cabinet == 2){
                client.publish('/cabinet/2',JSON.stringify({"release":1}))
            }
            break;
        case '/cabinet/1/sensor':
            resources.cabinets[0] = req.sensor
            break;
        case '/cabinet/2/sensor':
            resources.cabinets[1] = req.sensor
            break;
        case '/cabinet/detect':
            if(awaiting_release != -1){
                let o = resources.get_latest_order_id()

                resources.orders[o][awaiting_release] = resources.orders[o][awaiting_release] -1
                awaiting_release = -1
            }
    }
})

function release_one(order,o){
    let c = 0
    let k = Object.keys(order)
    let release = -1;
    for(let i=1;i<k.length+1;i++){
        if(order[i] > 0){
            release = i
            break
        }
    }

    if(release == -1){
        console.log("Nothing to release")
    }
    else{
        console.log("Releasing order",o, "Number",release)
        client.publish('/cabinet/'+String(release),JSON.stringify({"release":1}))
    }
    return release
}

function deleteEmpty(order,o){
    let k = Object.keys(order)
    let empty = true
    for(let i=1;i<k.length+1;i++){
        console.log(i,order[i])
        if(order[i] > 0){
            empty = false
            break
        }
    }
    if(empty){
        delete resources.orders[o]
    }
}


// Order task
setInterval(()=>{
    let o = resources.get_latest_order_id()
    o = resources.get_latest_order_id()
    if(o == -1){
        console.log("No more orders")
        return
    }
    o = resources.get_latest_order_id()
    deleteEmpty(resources.orders[o],o)
    o = resources.get_latest_order_id()
    if(o == -1){
        console.log("No more orders")
        return
    }
    if(awaiting_release == -1){
        let r = release_one(resources.orders[o],o)
        if(r != -1){
            awaiting_release = r
        }
    }
    else{
        console.log("Awaiting release",o,"Number",awaiting_release)
    }
},2000)

setInterval(()=>{
    client.publish('/cabinet/screen',JSON.stringify(cabinetRoutes.cabinet_screen()))
},2000)

const port = process.env.PORT || 5000
app.listen(port, ()=>{
    console.log("Listening on port "+port)
})