
#include <WiFi.h>
#include <PubSubClient.h>
#include <ArduinoJson.h>
#include <string.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include <ESP32Servo.h>


#define ssid "<SSID_HERE>"
#define password "<PASSWORD HERE>"


#define mqtt_server "broker.emqx.io"
#define mqtt_port 1883
#define mqtt_user ""
#define mqtt_password ""
#define mqtt_name "test_mqtt_cabinet_1"


int ldr1_read = 35;
int ldr2_read = 32;
int ldr3_read = 34;


WiFiClient espClient; // Initial client instance
PubSubClient client(espClient);
LiquidCrystal_I2C lcd(0x27, 16, 2);
Servo myservo;


void Connect_Wifi();


TaskHandle_t TaskA = NULL;


void setup(){
  Serial.begin(115200);
  myservo.attach(23);
  pinMode(22, OUTPUT);
  digitalWrite(22, LOW);
  pinMode(ldr1_read, INPUT);
  pinMode(ldr2_read, INPUT);
  pinMode(ldr3_read, INPUT);
  myservo.write(30);
 
  /* Connect to WiFi */
  Connect_Wifi();


  /* MQTT setup */
  client.setServer(mqtt_server, mqtt_port);
  client.setCallback(callback);
  if(client.connect(mqtt_name,mqtt_user,mqtt_password)){\
    client.subscribe("/cabinet/1");
    Serial.println("connected");
  }else{
    Serial.println("failed");
  }
 
  xTaskCreatePinnedToCore(Sensor, "Sensor", 2048, NULL, 1, &TaskA, 1);
}


void loop(){
 
}


void Connect_Wifi(){
    WiFi.begin(ssid, password);
    Serial.print("Connecting to WiFi");
    while (WiFi.status() != WL_CONNECTED)
    {
        delay(500);
        Serial.print(".");
    }
    Serial.print("OK! IP=");
    Serial.println(WiFi.localIP());
}


void callback(char* topic, byte* payload, unsigned int length){
  Serial.print("Message arrived [");
  Serial.print(topic); //ใน mqtx ต้องใส่ชื่อ topic ในช่อง topic
  Serial.print("] ");
  DynamicJsonDocument doc(2048);
  deserializeJson(doc, payload);
  int release = doc["release"];
  if(release != 0){
    for(int i = 0; i < release; i++){
      myservo.write(90);    
      delay(500);
      digitalWrite(22, HIGH);
      delay(1000);
      myservo.write(30);    
      delay(500);
      digitalWrite(22, LOW);
    }
  }
  Serial.print("release : ");
  Serial.println(release);


  /* Test */  
  // int activate = doc["order_id"];
  // Serial.print("order_ID : ");
  // Serial.println(activate);
  // int control = doc["control"];
  // Serial.print("control : ");
  // Serial.println(control);
  // if(activate == 0 && control == 0){
  //   myservo.write(0);    
  //   delay(500);
  //   digitalWrite(22, LOW);  
   
  // } else {
  //   myservo.write(90);  
  //   delay(500);    
  //   digitalWrite(22, HIGH);
  // }  
}  




void Sensor(void *parameter){
  while(1){
    int val_ldr1 = analogRead(ldr1_read);
    int val_ldr2 = analogRead(ldr2_read);
    int val_ldr3 = analogRead(ldr3_read);
    Serial.print("ldr1 = ");
    Serial.println(val_ldr1);
    Serial.print("ldr2 = ");
    Serial.println(val_ldr2);
    Serial.print("ldr3 = ");
    Serial.println(val_ldr3);
    Serial.println("--------------");
   
    if(val_ldr3 < 3000){ //2300
      send_mqtt("full");
    }else if (val_ldr2 < 2000){
      send_mqtt("half");
    }else if (val_ldr1 < 3700){
      send_mqtt("low");
    }else{
      send_mqtt("empty");  
    }
    vTaskDelay(1000/portTICK_PERIOD_MS);
  }
}


void send_mqtt(String status){
  String json;
  DynamicJsonDocument doc(2048);
  doc["sensor"] = status; //แก้
  serializeJson(doc, json);
  const char *json_char = json.c_str();
  if(client.connect(mqtt_name,mqtt_user,mqtt_password)){
    client.publish("/cabinet/1/sensor", json_char); // publish message ไปที่ /test/number2
  }else{
    Serial.println("failed");
  }    
  client.loop();
}